/******************** (C) COPYRIGHT 2008 PHYTEC Messtechnik ********************
* File Name          : UART.c
* Author             : Marc Dzierzawa
* Version            : V 0.1
* Date               : 06.01.09
* Description        : UART Functions
********************************************************************************/

#include "stm32f10x_lib.h"
#include "UART.h"


u8 InterpretFlag;
u8 UART1TxBuffer[UART1_BUFFERSIZE];
u8 UART1RxBuffer[UART1_BUFFERSIZE];
u8 TxBufferNextRead, TxBufferNumDat;
u8 RxBufferNumDat;

/*******************************************************************************
* Function Name  : I2C_EE_Init
* Description    : Init UART1 (Baud Rate: 9600, Data Bits:8, Stop Bits:1
				   Parity: Odd, Hardware Flow: None
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void UART1_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	USART_InitTypeDef USART_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	// Enable GPIOA and USART1 clocks
  	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_USART1, ENABLE);

	//Init USART1 Tx (PA9)
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  	GPIO_Init(GPIOA, &GPIO_InitStructure);

  	//Init USART1 Rx (PA10)
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  	GPIO_Init(GPIOA, &GPIO_InitStructure);

  /* Configure USART1:
        - Baud Rate = 9600 baud  
        - Word Length = 8 Bits
        - Two Stop Bit
        - Odd parity
        - Hardware flow control disabled (RTS and CTS signals)
  */
  USART_InitStructure.USART_BaudRate = 9600;
  USART_InitStructure.USART_WordLength = USART_WordLength_9b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_Odd;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

  //Init USART1
  USART_Init(USART1, &USART_InitStructure);

  // Enable USART1 Interrupt 
  NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQChannel;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  //Enable USART1 receive interrupt
  USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);

  // Enable USART1
  USART_Cmd(USART1, ENABLE);
  
  //Init buffer counters
   TxBufferNextRead=0; 
   TxBufferNumDat=0;
   RxBufferNumDat=0;
   InterpretFlag=0;
}

/*******************************************************************************
* Function Name  : UART1_SendString
* Description    : Store a String in UART1 Buffer to send it via UART1
* Input          : StringToSend: Pointer to String data to be sent
* Output         : None
* Return         : None
*******************************************************************************/
void UART1_SendString(u8* StringToSend)
{
	while(*StringToSend!=0)
	{
		while(UART1_SendByte(*StringToSend));
		StringToSend++;
	}
}
/*******************************************************************************
* Function Name  : UART1_SendByte
* Description    : Store byte in UART1 Buffer to send it via UART1
* Input          : ByteToSend: Byte to send over UART1
* Output         : 0 = Success
				   1 = buffer Overflow
* Return         : None
*******************************************************************************/
u8 UART1_SendByte(u8 ByteToSend)
{
	//Check for buffer overflow
	if(TxBufferNumDat>=(UART1_BUFFERSIZE-1))
		return 1;
	//Send data
	UART1TxBuffer[(TxBufferNextRead+TxBufferNumDat)%UART1_BUFFERSIZE]=ByteToSend;
	TxBufferNumDat++;
	//Enable Tx Interrupt
	if(TxBufferNumDat==1)
		USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
	return 0;
}

/*******************************************************************************
* Function Name  : USART1_IRQHandler
* Description    : This function handles USART1 global interrupt request.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void USART1_IRQHandler(void)
{
  u8 Buffer; 
  if(USART_GetITStatus(USART1, USART_IT_RXNE) != RESET)
  {
  	//Read received byte
 	Buffer = (USART_ReceiveData(USART1));
	if(!InterpretFlag)
	{
		if((RxBufferNumDat<(UART1_BUFFERSIZE-1))||(Buffer==0x0D))
		{
			if(Buffer==8) //Backspace
				RxBufferNumDat--;
			else
			{
				UART1RxBuffer[RxBufferNumDat]=Buffer;
				RxBufferNumDat++;
			}
			//Send back received byte
			USART_SendData(USART1, Buffer);
			USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
		}
		//In case of return send a '\n' and interpret the input  
		if(Buffer==0x0D)
		{
			UART1RxBuffer[RxBufferNumDat-1]=0;
			RxBufferNumDat=0;
			InterpretFlag=1;
			UART1_SendByte('\n');
		}
	}    
  }
   
  if(USART_GetITStatus(USART1, USART_IT_TXE) != RESET)
  {   
	if(TxBufferNumDat)
	{
		USART_SendData(USART1, UART1TxBuffer[TxBufferNextRead]);
		UART1TxBuffer[TxBufferNextRead]=0;
		TxBufferNextRead=(TxBufferNextRead+1)%UART1_BUFFERSIZE;
		TxBufferNumDat--;
	}                   
    //disable TX Interrupt if buffer is empty
	if(TxBufferNumDat==0)
		USART_ITConfig(USART1, USART_IT_TXE, DISABLE);
  }
}	

		
			
