/******************** (C) COPYRIGHT 2008 PHYTEC Messtechnik ********************
* File Name          : UARTShell.c
* Author             : Marc Dzierzawa
* Version            : V 0.1
* Date               : 28.01.09
* Description        : UARTShell Functions
********************************************************************************/

#include "stm32f10x_lib.h"
#include "baseboard.h"
#include "UARTShell.h"
#include "UART.h"

extern struct phy_GPIO Out[14];
extern struct phy_GPIO OutError[4];
extern struct phy_GPIO In[12];
extern struct phy_GPIO AOut[2];
extern struct phy_GPIO AIn[8];

extern u8 InterpretFlag;
extern u8 UART1RxBuffer[UART1_BUFFERSIZE];


/*******************************************************************************
* Function Name  : UARTShell_Interpret
* Description    : Interprets the UART Input
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void UARTShell_Interpret(void)
{
	s32 param;
	u8 buffer[12];
	u32 temp;
	u8 i; 

	if(InterpretFlag)
	{
		if(UARTShell_Cmp_String(UART1RxBuffer,"SetOut"))
		{
			param=UARTShell_ASCII_to_Hex(UART1RxBuffer[7]);
			if((param!=-1) && (param<14))
			{
				if(param<6)
					GPIO_SetBits(Out[param].Port,Out[param].Pin); 	//Relays, high active
				else
					GPIO_ResetBits(Out[param].Port,Out[param].Pin);	//Transistors, low active
			}	
			else
				UART1_SendString("Unknown Parameter\r\n");
		}

		else if(UARTShell_Cmp_String(UART1RxBuffer,"ResetOut"))
		{
			param=UARTShell_ASCII_to_Hex(UART1RxBuffer[9]);
			if((param!=-1) && (param<14))
			{
				if(param<6)
					GPIO_ResetBits(Out[param].Port,Out[param].Pin);	//Relays, high active
				else
					GPIO_SetBits(Out[param].Port,Out[param].Pin); 	//Relays, low active
			}
			else
				UART1_SendString("Unknown Parameter\r\n");
			
		}

		else if(UARTShell_Cmp_String(UART1RxBuffer,"GetIn"))
		{
			
			param=UARTShell_ASCII_to_Hex(UART1RxBuffer[6]);
		    if((param!=-1)&&(param!=8) &&(param<12))  //Input 8 is not used
			{
				UART1_SendString("Value: ");
				UART1_SendByte(49-GPIO_ReadInputDataBit(In[param].Port, In[param].Pin));
				UART1_SendString("\r\n");
			}
			else
				UART1_SendString("Unknown Parameter\r\n");
		}

		else if (UARTShell_Cmp_String(UART1RxBuffer,"GetAllIn"))	
		{
			temp=0;
			for(i=0;i<12;i++)
			{
				if(i!=8)
					temp=temp+((1-GPIO_ReadInputDataBit(In[i].Port, In[i].Pin))<<i);
				
			}
			if(UARTShell_Hex_to_ASCII(temp, buffer))
			{
				UART1_SendString("Value: ");
				UART1_SendString(buffer);
				UART1_SendString("\r\n");
			}
			else
				UART1_SendString("Error in Hex_to_ASCII!\r\n");
		}
		  
		else if (UARTShell_Cmp_String(UART1RxBuffer,"GetOutError"))
		{
			temp=0;
			for(i=0;i<4;i++)
			{
				if(!GPIO_ReadInputDataBit(OutError[i].Port, OutError[i].Pin))
				{
					if(temp>0)
					{
						buffer[temp]=',';
						temp++;
					}
					buffer[temp]=i+48;
					temp++;
				}
			}
			buffer[temp]=0;
			if(temp>0)
			{
				UART1_SendString("Error in Channel:");
				UART1_SendString(buffer);
				UART1_SendString("\r\n");
			}
			else
				UART1_SendString("No Errors in Ouput Channel\r\n");
		}

		else if (UARTShell_Cmp_String(UART1RxBuffer,"GetAIn"))
		{
			param=UARTShell_ASCII_to_Hex(UART1RxBuffer[7]);
		    if((param!=-1)&&(param<6)) 
			{
				if(UARTShell_Interpret_AIn(Get_Analog_In(param), buffer))
				{
					UART1_SendString("Value: ");
					UART1_SendString(buffer);
					UART1_SendString("\r\n");
				}
				else
					UART1_SendString("Error in Hex_to_ASCII!\r\n");
			}
			else
				UART1_SendString("Unknown Parameter\r\n");
		
		}

		else if (UARTShell_Cmp_String(UART1RxBuffer,"SetAOut"))	
		{
			temp=0;
			param=UART1RxBuffer[8];
			if((param=='0')||(param=='1'))
			{
				for(i=0;i<3;i++)
				{
					if(UART1RxBuffer[i+10]==0)
						i=3;
					else
					{
						param=UARTShell_ASCII_to_Hex(UART1RxBuffer[i+10]);
						if(param!=-1)
							temp=temp+(param<<(4*(2-i)));
						else
							temp=temp+0xF000;
					}
				}
				if(temp<=0xFFF)
				{
					UARTShell_Hex_to_ASCII(temp, buffer);
					if(UART1RxBuffer[8]=='0')
					{
						DAC_SetChannel1Data(DAC_Align_12b_R, temp);
  						DAC_SoftwareTriggerCmd(DAC_Channel_1, ENABLE);
						UART1_SendString("DAC 0 set to: ");
						UART1_SendString(buffer);
						UART1_SendString("\r\n");

					}
					else
					{
						DAC_SetChannel2Data(DAC_Align_12b_R, temp);
  						DAC_SoftwareTriggerCmd(DAC_Channel_2, ENABLE);
						UART1_SendString("DAC 1 set to: ");
						UART1_SendString(buffer);
						UART1_SendString("\r\n");
					}
				}
				else
					UART1_SendString("Error in SetDAC!\r\n");
			}
			else
				UART1_SendString("No such DAC\r\n");
						
		}


		else
			UART1_SendString("Unknown Command\r\n");
		InterpretFlag=0;
	}
}

/*******************************************************************************
* Function Name  : UARTShell_Cmp_String
* Description    : Compares two strings
* Input          : *string1, *string2 : pointer to a string
* Output         : None
* Return         : 0: Strings are not equal
				   1: Strings are equal
*******************************************************************************/
u8 UARTShell_Cmp_String(u8* string1, u8* string2)
{
	u8 i=0;
	while((i<UART1_BUFFERSIZE)&& (string1[i]!=0)&&(string2[i]!=0))
	{
		if(string1[i]!=string2[i])
			return 0;
		i++;
	}	  
	return 1;
}

/*******************************************************************************
* Function Name  : UARTShell_ASCII_to_Hex
* Description    : Interprets an ASCII char as hex
* Input          : u8 inp: input char
* Output         : None
* Return         : -1: Error, char is not a Hex number
				   0-15: Hex number
*******************************************************************************/
s8 UARTShell_ASCII_to_Hex(u8 inp)
{
	if((inp>=48)&&(inp<=57))		//0-9
		inp=inp-48;
	else if ((inp>=65)&&(inp<=70))
		inp=inp-55;
	else
		return -1;
	return inp;
}

/*******************************************************************************
* Function Name  : UARTShell_Hex_to_ASCII
* Description    : Returns the correspondent ASCII value of a Hex number
* Input          : u32 inp: input Hex number
* Output         : u8* : pointer to output array
* Return         : 0: Error during conversion
				   1: conversion successful
*******************************************************************************/
u8 UARTShell_Hex_to_ASCII(u32 inp, u8* out)
{
	u8 i=0;
	u8 temp;
	out[0]='0';
	out[1]='x';
	for(i=0;i<8;i++)
	{
		temp=inp&0xF;
		if(temp<10)
			out[9-i]=temp+48;
		else if ((temp>10)||(temp<16))
			out[9-i]=temp+55;
		else 
			return 0;
		inp=inp>>4;
	}
	out[10]=0;
	return 1;
}

/*******************************************************************************
* Function Name  : UARTShell_Interpret_AIn
* Description    : Interprets the value of an AIn
* Input          : u32 inp: input value
* Output         : u8* : pointer to output array
* Return         : 0: Error during conversion
				   1: conversion successful
*******************************************************************************/
u8 UARTShell_Interpret_AIn(u32 inp, u8* out)
{
	float digit,value;
	u8 i;

	digit=10.0/4095;
	value=digit*inp;
	if(value>=10)
	{
		out[0]='1';
		out[1]='0';
		out[2]='V';
		out[3]=0;
	}
	else
	{
		for(i=0;i<5;i++)
		{
			out[i]=(u8)value+48;
			value=value-(u8)value;
			value=value*10;
			if(i==0)
			{
				i++;
				out[i]='.';
			}
		}
		out[4]='V';
		out[5]=0;
	}
	return 1;
}	
			
	
		

