/******************** (C) COPYRIGHT 2008 PHYTEC Messtechnik ********************
* File Name          : baseboard.c
* Author             : Marc Dzierzawa
* Version            : V 0.1
* Date               : 28.01.08
* Description        : Baseboard IO Functions
********************************************************************************/

#include "stm32f10x_lib.h"
#include "baseboard.h"
#include "can.h"
#include "UART.h"

struct phy_GPIO Out[14];
struct phy_GPIO OutError[4];
struct phy_GPIO In[12];
struct phy_GPIO AOut[2];
struct phy_AIN AIn[8];


/*******************************************************************************
* Function Name  : Init_GPIO_Structs
* Description    : Init the structs wich will be used to initialize the GPIOs 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void Init_GPIO_Structs(void)
{
	//Init Array for digital Outsputs
	//First Relays
	Out[0].Port=GPIOD;
	Out[0].Pin=GPIO_Pin_2;
	Out[1].Port=GPIOD;
	Out[1].Pin=GPIO_Pin_3;
	Out[2].Port=GPIOD;
	Out[2].Pin=GPIO_Pin_6;
	Out[3].Port=GPIOF;
	Out[3].Pin=GPIO_Pin_11;
	Out[4].Port=GPIOG;
	Out[4].Pin=GPIO_Pin_6;
	Out[5].Port=GPIOG;
	Out[5].Pin=GPIO_Pin_7;

	//Transistors
	Out[6].Port=GPIOG;
	Out[6].Pin=GPIO_Pin_8;
	Out[7].Port=GPIOG;
	Out[7].Pin=GPIO_Pin_10;
	Out[8].Port=GPIOG;
	Out[8].Pin=GPIO_Pin_11;
	Out[9].Port=GPIOG;
	Out[9].Pin=GPIO_Pin_12;
	Out[10].Port=GPIOG;
	Out[10].Pin=GPIO_Pin_13;
	Out[11].Port=GPIOF;
	Out[11].Pin=GPIO_Pin_9;
	Out[12].Port=GPIOG;
	Out[12].Pin=GPIO_Pin_14;
	Out[13].Port=GPIOF;
	Out[13].Pin=GPIO_Pin_10;

	//Init Array for Transistor Out Error Inputs
	OutError[0].Port=GPIOA;
	OutError[0].Pin=GPIO_Pin_8;
	OutError[1].Port=GPIOB;
	OutError[1].Pin=GPIO_Pin_5;
	OutError[2].Port=GPIOB;
	OutError[2].Pin=GPIO_Pin_6;
	OutError[3].Port=GPIOB;
	OutError[3].Pin=GPIO_Pin_7;

	//Init Array for digital Inputs
	In[0].Port=GPIOC;
	In[0].Pin=GPIO_Pin_10;
	In[1].Port=GPIOC;
	In[1].Pin=GPIO_Pin_11;
	In[2].Port=GPIOC;
	In[2].Pin=GPIO_Pin_12;
	In[3].Port=GPIOC;
	In[3].Pin=GPIO_Pin_13;
	In[4].Port=GPIOC;
	In[4].Pin=GPIO_Pin_14;
	In[5].Port=GPIOC;
	In[5].Pin=GPIO_Pin_15;
	In[6].Port=GPIOD;
	In[6].Pin=GPIO_Pin_7;
	In[7].Port=GPIOE;
	In[7].Pin=GPIO_Pin_2;
	In[8].Port=GPIOE;
	In[8].Pin=GPIO_Pin_3;
	In[9].Port=GPIOE;
	In[9].Pin=GPIO_Pin_4;
	In[10].Port=GPIOE;
	In[10].Pin=GPIO_Pin_5;
	In[11].Port=GPIOE;
	In[11].Pin=GPIO_Pin_6;

	//Init Array for analog Outputs
	AOut[0].Port=GPIOA;
	AOut[0].Pin=GPIO_Pin_4;
	AOut[1].Port=GPIOA;
	AOut[1].Pin=GPIO_Pin_5;

	//Init Array for analog Inputs
	AIn[0].GPIO.Port=GPIOC;
	AIn[0].GPIO.Pin=GPIO_Pin_3;
	AIn[0].ADC=ADC1;
	AIn[0].ADC_Channel=ADC_Channel_13;
	AIn[1].GPIO.Port=GPIOC;
	AIn[1].GPIO.Pin=GPIO_Pin_4;
	AIn[1].ADC=ADC1;
	AIn[1].ADC_Channel=ADC_Channel_14;
	AIn[2].GPIO.Port=GPIOC;
	AIn[2].GPIO.Pin=GPIO_Pin_5;
	AIn[2].ADC=ADC1;
	AIn[2].ADC_Channel=ADC_Channel_15;
	AIn[3].GPIO.Port=GPIOF;
	AIn[3].GPIO.Pin=GPIO_Pin_6;
	AIn[3].ADC=ADC3;
	AIn[3].ADC_Channel=ADC_Channel_4;
	AIn[4].GPIO.Port=GPIOF;
	AIn[4].GPIO.Pin=GPIO_Pin_7;
	AIn[4].ADC=ADC3;
	AIn[4].ADC_Channel=ADC_Channel_5;
	AIn[5].GPIO.Port=GPIOF;
	AIn[5].GPIO.Pin=GPIO_Pin_8;
	AIn[5].ADC=ADC3;
	AIn[5].ADC_Channel=ADC_Channel_6;
	AIn[6].GPIO.Port=GPIOF;
	AIn[6].GPIO.Pin=GPIO_Pin_9;
	AIn[6].ADC=ADC3;
	AIn[6].ADC_Channel=ADC_Channel_7;
	AIn[7].GPIO.Port=GPIOF;
	AIn[7].GPIO.Pin=GPIO_Pin_10;
	AIn[7].ADC=ADC3;
	AIn[7].ADC_Channel=ADC_Channel_8;

}

/*******************************************************************************
* Function Name  : Init_Rel_Out
* Description    : Init the GPIOs for Output 0-5 (Relays)
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void Init_Rel_Out(void)
{
  int i;
  GPIO_InitTypeDef GPIO_InitStructure;

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  for(i=0;i<6;i++)
  {
  	GPIO_InitStructure.GPIO_Pin = Out[i].Pin;
  	GPIO_Init(Out[i].Port, &GPIO_InitStructure);
  }

  
}

/*******************************************************************************
* Function Name  : Init_Tran_Out
* Description    : Init the GPIOs for Output 6-13 (Transistor)
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void Init_Tran_Out(void)
{
   int i;
   GPIO_InitTypeDef GPIO_InitStructure;

   //First set the GPIOs to 1 so switch of off the outputs
   for(i=6;i<14;i++)
   		GPIO_SetBits(Out[i].Port, Out[i].Pin);

    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
   	for(i=6;i<14;i++)
   	{
   		GPIO_InitStructure.GPIO_Pin = Out[i].Pin;
  		GPIO_Init(Out[i].Port, &GPIO_InitStructure);
	}


  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  	for(i=0;i<4;i++)
   	{
   		GPIO_InitStructure.GPIO_Pin = OutError[i].Pin;
  		GPIO_Init(OutError[i].Port, &GPIO_InitStructure);
	}
  
}

/*******************************************************************************
* Function Name  : Init_Digital_In
* Description    : Init the GPIOs for digital Input 0-11 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void Init_Digital_In(void)
{
  	int i;
  	GPIO_InitTypeDef GPIO_InitStructure;

  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  	for(i=0;i<12;i++)
  	{
  		if(i!=8)	//if 2MB external SRAM is used
		{
  			GPIO_InitStructure.GPIO_Pin = In[i].Pin;
  			GPIO_Init(In[i].Port, &GPIO_InitStructure);
		}
	}
}

/*******************************************************************************
* Function Name  : Init_Analog_In
* Description    : Init the GPIOs for analog Input 0-7 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void Init_Analog_In(void)
{
  	u8 i;
  	GPIO_InitTypeDef GPIO_InitStructure;
	ADC_InitTypeDef   ADC_InitStructure;

  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
   	for(i=0;i<6;i++)
	{
  		GPIO_InitStructure.GPIO_Pin = AIn[i].GPIO.Pin;
		GPIO_Init(AIn[i].GPIO.Port, &GPIO_InitStructure);
	}

	//Init ADC1 and ADC3
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1 | RCC_APB2Periph_ADC3, ENABLE);
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
  	ADC_InitStructure.ADC_ScanConvMode = DISABLE;
  	ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
  	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = 1;
  	ADC_Init(ADC1, &ADC_InitStructure);
	ADC_Init(ADC3, &ADC_InitStructure);


	ADC_Cmd(ADC1, ENABLE); 
  	ADC_ResetCalibration(ADC1);
  	while(ADC_GetResetCalibrationStatus(ADC1));
  	ADC_StartCalibration(ADC1);
  	while(ADC_GetCalibrationStatus(ADC1));

	ADC_Cmd(ADC3, ENABLE);
  	ADC_ResetCalibration(ADC3);
  	while(ADC_GetResetCalibrationStatus(ADC3));
  	ADC_StartCalibration(ADC3);
  	while(ADC_GetCalibrationStatus(ADC3));
	  	
	UART1_SendString("ADC initialized successful\r\n"); 
}

/*******************************************************************************
* Function Name  : Get_Analog_In
* Description    : Gets the value of the specified Analog Input
* Input          : u8 Analog Input number
* Output         : None
* Return         : Converted Value
*******************************************************************************/
u16 Get_Analog_In(u8 numAIn)
{
	ADC_RegularChannelConfig(AIn[numAIn].ADC, AIn[numAIn].ADC_Channel, 1, ADC_SampleTime_28Cycles5);   	

	ADC_SoftwareStartConvCmd(AIn[numAIn].ADC,ENABLE);
	while (!ADC_GetFlagStatus(AIn[numAIn].ADC,ADC_FLAG_EOC));
	return ADC_GetConversionValue(AIn[numAIn].ADC); 
}

/*******************************************************************************
* Function Name  : Init_Analog_Out
* Description    : Init the GPIOs for analog Output 0-1 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void Init_Analog_Out(void)
{
  	u8 i;
  	GPIO_InitTypeDef GPIO_InitStructure;
	DAC_InitTypeDef DAC_InitStructure;

  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  
  	for(i=0;i<2;i++)
	{
		GPIO_InitStructure.GPIO_Pin =  AOut[i].Pin;
  		GPIO_Init(AOut[i].Port, &GPIO_InitStructure);
	}
  
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_DAC, ENABLE);

  // DAC channel1 Configuration 
  DAC_InitStructure.DAC_Trigger = DAC_Trigger_Software;
  DAC_InitStructure.DAC_WaveGeneration = DAC_WaveGeneration_None;
  DAC_InitStructure.DAC_OutputBuffer = DAC_OutputBuffer_Disable;
  DAC_Init(DAC_Channel_1, &DAC_InitStructure);
  DAC_Cmd(DAC_Channel_1, ENABLE);
  DAC_SetChannel1Data(DAC_Align_12b_R, 0x000);
  DAC_SoftwareTriggerCmd(DAC_Channel_1, ENABLE);


  // DAC channel2 Configuration 
  DAC_Init(DAC_Channel_2, &DAC_InitStructure);
  DAC_Cmd(DAC_Channel_2, ENABLE);
  DAC_SetChannel2Data(DAC_Align_12b_R, 0x000);
  DAC_SoftwareTriggerCmd(DAC_Channel_2, ENABLE);
}  


/*******************************************************************************
* Function Name  : SevenSeg_Write
* Description    : Writes Data to the 7-Segment Displays
* Input          : u8 data display1, u8 data display 2
* Output         : None
* Return         : None
*******************************************************************************/

void SevenSeg_Write(u8 data1, u8 data2)
{
	u16 data;
	//the databus must be used, so it is absolut necessary the the external SRAM isn't be used!
	//first process the input data (data2=LSB => left display, data1=MSB => right display)
	data=~(data2+(data1<<8));
	//Set PC9 to on 
	GPIO_SetBits(GPIOC, GPIO_Pin_9);
	//Write to a not accessible Adress in external SRAM
	*(u32*)0x64000000=data;
	//Set PC9 directly to off 
	 GPIOC->BRR = GPIO_Pin_9;
	
}

/*******************************************************************************
* Function Name  : Init_Baseboard
* Description    : Init the GPIOs for all In/Outputs 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void Init_Baseboard(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

    //Init GPIO for 7-Seg. enable
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
  	GPIO_Init(GPIOC, &GPIO_InitStructure);


	Init_GPIO_Structs();
	Init_Rel_Out();
	Init_Tran_Out();
	Init_Digital_In();
	Init_Analog_In();
	Init_Analog_Out();
	SevenSeg_Write(0x54,0x3F);

	if(CAN_Bus_Init())
		UART1_SendString("CAN Bus initialized successful\r\n");
	else
		UART1_SendString("ERROR: CAN Bus initialization failed!\r\n"); 

	//Init USB GPIO as Pullup
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
  	GPIO_Init(GPIOA, &GPIO_InitStructure);
	GPIO_SetBits(GPIOA, GPIO_Pin_12);

}
	



